/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2021/06/06
 * Description        : Main program body.
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/

/*
 *@Note
 APPʾ̣
    FLASH  ROM - 64K  RAM - 20K
        ûFLASH-64K - öǰ4KĬд
        IAP -  0x08000000 - 8K
        APP -  0x08002000 - 24K
        APP_BUF - 0x08008000 - 24K
        dataflash - 籣 256B - 0x0800FF00
 ע
 1. MCU CH32V203C8T6
 2. CANID 0x316
 
 */

#include "debug.h"

#define VERSION  0x10  //V1.0

/* FLASH䶨  */
#define ADDR_IAP       0x08000000
#define ADDR_APP       0x08002000
#define ADDR_APP_BUF   0x08008000

/*ǰ4ֽǲͷ@ W C H
        5ֽжARM/RVδ
        6ֽжAPP/IAP0x5A-APPڣ-APP
*/
#define ADDR_IAP_APP       0x0800FF00


#define CAN_ID       0x317
#define CAN_ID_USB   0x316

const u8 Verify[4]={'@','W','C','H'};
u8 Status[6];
u8 Flag_verifysuc =0;  //1IAPɹ
u8 Flag_firstIAP;
u8 CAN_buf[8];
__attribute__ ((aligned(4))) u8 ReadData_DBG_256[256];



/*******************************************************************************
* Function Name  : FlashReadBuf
* Description    : Flashdatacode
* Input          :
* Output         : None
* Return         : None
*******************************************************************************/
void FlashReadBuf(u32 Addr,u8* buf,u16 len)
{
    u16 i;

    for(i=0; i!=len; i++)
    {
        buf[i] = *(u8*)Addr;
        Addr++;
    }
}

/*********************************************************************
 * @fn      FLASH_CFGFLAG_PR
 *
 * @brief   IAPлRV-RAMģʽл ־
 *
 * @param
 *
 * @return  none
 */
void FLASH_CFGFLAG_PR(u32 addr, u8* buf, u8 len )
{
    u16 i;
    u32 addr1;

    addr1 = addr&(~0xFF);
    for(i=0; i<256; i++){
        ReadData_DBG_256[i] = *(u8*)(addr1+i);
    }

    for(i=0; i<len; i++){
        ReadData_DBG_256[i+addr-addr1] = buf[i];
    }

    /* FLASH OP */
    FLASH_Unlock_Fast();
    FLASH_ErasePage_Fast(addr1);
    FLASH_ProgramPage_Fast(addr1, (u32*)ReadData_DBG_256);
    FLASH_Lock_Fast();
}
/*********************************************************************
 * @fn      CAN_Mode_Init
 *
 * @brief   Initializes CAN communication test mode.
 *          Bps =Fpclk1/((tpb1+tbs2+1)*brp)
 *
 * @param   tsjw - CAN synchronisation jump width.
 *          tbs2 - CAN time quantum in bit segment 1.
 *          tbs1 - CAN time quantum in bit segment 2.
 *          brp - Specifies the length of a time quantum.
 *          mode - Test mode.
 *            CAN_Mode_Normal.
 *            CAN_Mode_LoopBack.
 *            CAN_Mode_Silent.
 *            CAN_Mode_Silent_LoopBack.
 *
 * @return  none
 */
void CAN_Mode_Init(u8 tsjw, u8 tbs2, u8 tbs1, u16 brp, u8 mode)
{
    GPIO_InitTypeDef      GPIO_InitSturcture = {0};
    CAN_InitTypeDef       CAN_InitSturcture = {0};
    CAN_FilterInitTypeDef CAN_FilterInitSturcture = {0};
    NVIC_InitTypeDef NVIC_InitStructure = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);

    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitSturcture.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    CAN_InitSturcture.CAN_TTCM = DISABLE;
    CAN_InitSturcture.CAN_ABOM = DISABLE;
    CAN_InitSturcture.CAN_AWUM = DISABLE;
    CAN_InitSturcture.CAN_NART = ENABLE;
    CAN_InitSturcture.CAN_RFLM = DISABLE;
    CAN_InitSturcture.CAN_TXFP = DISABLE;
    CAN_InitSturcture.CAN_Mode = mode;
    CAN_InitSturcture.CAN_SJW = tsjw;
    CAN_InitSturcture.CAN_BS1 = tbs1;
    CAN_InitSturcture.CAN_BS2 = tbs2;
    CAN_InitSturcture.CAN_Prescaler = brp;
    CAN_Init(CAN1, &CAN_InitSturcture);

    CAN_FilterInitSturcture.CAN_FilterNumber = 0;

    /* identifier list mode, One 32-bit filter, StdId: 0x3170x316 */
    CAN_FilterInitSturcture.CAN_FilterMode = CAN_FilterMode_IdList;
    CAN_FilterInitSturcture.CAN_FilterScale = CAN_FilterScale_32bit;
    CAN_FilterInitSturcture.CAN_FilterIdHigh = 0x62E0;
    CAN_FilterInitSturcture.CAN_FilterIdLow = 0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdHigh = 0x62C0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdLow = 0;

    CAN_FilterInitSturcture.CAN_FilterFIFOAssignment = CAN_Filter_FIFO0;
    CAN_FilterInitSturcture.CAN_FilterActivation = ENABLE;
    CAN_FilterInit(&CAN_FilterInitSturcture);

    CAN_ClearITPendingBit( CAN1, CAN_IT_FMP0 );

    NVIC_InitStructure.NVIC_IRQChannel = USB_LP_CAN1_RX0_IRQn;

    NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
    NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
    NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
    NVIC_Init(&NVIC_InitStructure);
    CAN_ITConfig( CAN1, CAN_IT_FMP0, ENABLE );      //FIFO0յһ±ж
}

/*********************************************************************
 * @fn      CAN_Send_Msg
 *
 * @brief   CAN Transmit function.
 *
 * @param   msg - Transmit data buffer.
 *          len - Data length.
 *
 * @return  0 - Send successful.
 *          1 - Send failed.
 */
u8 CAN_Send_Msg(u8 *msg, u8 len)
{
    u8  mbox;
    u16 i = 0;
    CanTxMsg CanTxStructure;

    CanTxStructure.StdId = CAN_ID;
    CanTxStructure.IDE = CAN_Id_Standard;

    CanTxStructure.RTR = CAN_RTR_Data;
    CanTxStructure.DLC = len;

    for(i = 0; i < len; i++) {
        CanTxStructure.Data[i] = msg[i];
    }

    mbox = CAN_Transmit(CAN1, &CanTxStructure);
    i = 0;

    while((CAN_TransmitStatus(CAN1, mbox) != CAN_TxStatus_Ok) && (i < 0xFFF))
    {
        i++;
    }

    if(i == 0xFFF)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}


/*********************************************************************
 * @fn      CAN_Rx_Deal
 *
 * @brief   CAN Receive function.
 *
 * @param   buf - Receive data buffer.
 *          len - Data length.
 *
 * @return  CanRxStructure.DLC - Receive data length.
 */
u8 CAN_Rx_Deal(u8 *buf)
{
    u8 i, s;

    CanRxMsg CanRxStructure;

    while(CAN_MessagePending(CAN1, CAN_FIFO0) == 0)
    {
    }
    CAN_Receive(CAN1, CAN_FIFO0, &CanRxStructure);

    for(i = 0; i < 8; i++) {
        buf[i] = CanRxStructure.Data[i];
    }
    if((CanRxStructure.StdId == CAN_ID_USB) && (CanRxStructure.Data[0] == 0x50)){ /* reset to IAP */
        for(i=0;i<4;i++)
               {
                   Status[i]=Verify[i];
               }
        Status[5] = 0xA5;
        FLASH_CFGFLAG_PR( ADDR_IAP_APP, Status, 6);
        PR("jump IAP\r\n");
        Delay_Ms(50);
        NVIC_SystemReset();
    }
    else if((CanRxStructure.StdId == CAN_ID_USB) && (CanRxStructure.Data[0] == 0x51)){ /* Get Version */
        CAN_buf[0] = 0;
        CAN_buf[1] = VERSION;
        CAN_Send_Msg(CAN_buf, 2);
    }

    return s;
}

void USB_LP_CAN1_RX0_IRQHandler(void) __attribute__((interrupt("WCH-Interrupt-fast")));

/*********************************************************************
 * @fn      USB_LP_CAN1_RX0_IRQHandler
 *
 * @brief   This function handles Hard Fault exception.
 *
 * @return  none
 */
void USB_LP_CAN1_RX0_IRQHandler(void)
{
    if( CAN_GetITStatus( CAN1, CAN_IT_FMP0 ) != RESET ){
        CAN_Rx_Deal(CAN_buf);
    }

    CAN_ClearITPendingBit( CAN1, CAN_IT_FMP0 );
}

/*********************************************************************
 * @fn      main
 *
 * @brief   Main program.
 *
 * @return  none
 */
int main(void)
{
    Delay_Init();
    USART_Printf_Init(115200);
    PR("SystemClk:%d\r\n", SystemCoreClock);
    PR("APP5\r\n");

    /* Bps = 250Kbps */
    CAN_Mode_Init(CAN_SJW_1tq, CAN_BS2_5tq, CAN_BS1_6tq, 16, CAN_Mode_Normal);
    while(1)
    {
    }
}
