/********************************** (C) COPYRIGHT  *******************************
 * File Name          : iap.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2020/12/16
 * Description        : IAP
 *  Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
#include "iap.h"
#include "string.h"
#include "core_riscv.h"

/******************************************************************************/

#define USBD_DATA_SIZE               64
u32 Program_addr;
u32 Verity_addr;
u8 Verity_Star_flag = 0;
u8 Fast_Program_Buf[390];
u32 CodeLen = 0;
u8 EP2_Rx_Buffer[USBD_DATA_SIZE];
#define  isp_cmd_t   ((isp_cmd  *)EP2_Rx_Buffer)

u16 Bin_Len;  //用户代码总长度
u32 CRC_RxVy; //接收完整Bin文件，硬件CRC校验值

const u8 Verify[4]={'@','W','C','H'};
u8 Status[6];
u8 Flag_verifysuc =0;  //1：IAP升级成功
u8 Flag_firstIAP;


/*********************************************************************
 * @fn      CAN_Mode_Init
 *
 * @brief   Initializes CAN communication test mode.
 *          Bps =Fpclk1/((tpb1+tbs2+1)*brp)
 *
 * @param   tsjw - CAN synchronisation jump width.
 *          tbs2 - CAN time quantum in bit segment 1.
 *          tbs1 - CAN time quantum in bit segment 2.
 *          brp - Specifies the length of a time quantum.
 *          mode - Test mode.
 *            CAN_Mode_Normal.
 *            CAN_Mode_LoopBack.
 *            CAN_Mode_Silent.
 *            CAN_Mode_Silent_LoopBack.
 *
 * @return  none
 */
void CAN_Mode_Init(u8 tsjw, u8 tbs2, u8 tbs1, u16 brp, u8 mode)
{
    GPIO_InitTypeDef      GPIO_InitSturcture = {0};
    CAN_InitTypeDef       CAN_InitSturcture = {0};
    CAN_FilterInitTypeDef CAN_FilterInitSturcture = {0};

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO | RCC_APB2Periph_GPIOB, ENABLE);
    RCC_APB1PeriphClockCmd(RCC_APB1Periph_CAN1, ENABLE);

    GPIO_PinRemapConfig(GPIO_Remap1_CAN1, ENABLE);

    RCC_APB1PeriphResetCmd(RCC_APB1Periph_CAN1, ENABLE);
    RCC_APB1PeriphResetCmd(RCC_APB1Periph_CAN1, DISABLE);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_9;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_AF_PP;
    GPIO_InitSturcture.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    GPIO_InitSturcture.GPIO_Pin = GPIO_Pin_8;
    GPIO_InitSturcture.GPIO_Mode = GPIO_Mode_IPU;
    GPIO_Init(GPIOB, &GPIO_InitSturcture);

    CAN_InitSturcture.CAN_TTCM = DISABLE;
    CAN_InitSturcture.CAN_ABOM = DISABLE;
    CAN_InitSturcture.CAN_AWUM = DISABLE;
    CAN_InitSturcture.CAN_NART = ENABLE;
    CAN_InitSturcture.CAN_RFLM = DISABLE;
    CAN_InitSturcture.CAN_TXFP = DISABLE;
    CAN_InitSturcture.CAN_Mode = mode;
    CAN_InitSturcture.CAN_SJW = tsjw;
    CAN_InitSturcture.CAN_BS1 = tbs1;
    CAN_InitSturcture.CAN_BS2 = tbs2;
    CAN_InitSturcture.CAN_Prescaler = brp;
    CAN_Init(CAN1, &CAN_InitSturcture);

    CAN_FilterInitSturcture.CAN_FilterNumber = 0;

    /* identifier list mode, One 32-bit filter, StdId: 0x317，0x316 */
    CAN_FilterInitSturcture.CAN_FilterMode = CAN_FilterMode_IdList;
    CAN_FilterInitSturcture.CAN_FilterScale = CAN_FilterScale_32bit;
    CAN_FilterInitSturcture.CAN_FilterIdHigh = 0x62E0;
    CAN_FilterInitSturcture.CAN_FilterIdLow = 0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdHigh = 0x62C0;
    CAN_FilterInitSturcture.CAN_FilterMaskIdLow = 0;

    CAN_FilterInitSturcture.CAN_FilterFIFOAssignment = CAN_Filter_FIFO0;
    CAN_FilterInitSturcture.CAN_FilterActivation = ENABLE;
    CAN_FilterInit(&CAN_FilterInitSturcture);

}

/*********************************************************************
 * @fn      CAN_Send_Msg
 *
 * @brief   CAN Transmit function.
 *
 * @param   msg - Transmit data buffer.
 *          len - Data length.
 *
 * @return  0 - Send successful.
 *          1 - Send failed.
 */
u8 CAN_Send_Msg(u8 *msg, u8 len)
{
    u8  mbox;
    u16 i = 0;

    CanTxMsg CanTxStructure;

    CanTxStructure.StdId = CAN_ID;
    CanTxStructure.IDE = CAN_Id_Standard;

    CanTxStructure.RTR = CAN_RTR_Data;
    CanTxStructure.DLC = len;

    for(i = 0; i < len; i++) {
        CanTxStructure.Data[i] = msg[i];
    }

    mbox = CAN_Transmit(CAN1, &CanTxStructure);
    i = 0;

    while((CAN_TransmitStatus(CAN1, mbox) != CAN_TxStatus_Ok) && (i < 0xFFF))
    {
        i++;
    }

    if(i == 0xFFF)
    {
        return 1;
    }
    else
    {
        return 0;
    }
}

/*********************************************************************
 * @fn      CAN_Receive_Msg
 *
 * @brief   CAN Receive function.
 *
 * @param   buf - Receive data buffer.
 *          len - Data length.
 *
 * @return  CanRxStructure.DLC - Receive data length.
 */
u8 CAN_Receive_Msg(u8 *buf)
{
    u8 i;

    CanRxMsg CanRxStructure;
    while(((uint8_t)(CAN1->RFIFO0&(uint32_t)0x03)) == 0)
    {
    }
    CAN_Receive(CAN1, CAN_FIFO0, &CanRxStructure);

    for(i = 0; i < 8; i++) {
        buf[i] = CanRxStructure.Data[i];
    }

    return CanRxStructure.DLC;
}

/*********************************************************************
 * @fn      CRC_Verify_Init
 *
 * @brief
 *
 * @return
 */
void CRC_Verify_Init(void)
{
    RCC_AHBPeriphClockCmd(RCC_AHBPeriph_CRC, ENABLE);
    CRC_ResetDR();
}

/*********************************************************************
 * @fn      IAP_2_APP
 *
 * @brief
 *
 * @return  none
 */
void IAP_2_APP(void)
{
    // 进 机器模式 跳转到 APP;
    NVIC_EnableIRQ(Software_IRQn);
    NVIC_SetPendingIRQ(Software_IRQn);

}


/*******************************************************************************
* Function Name  : FlashReadBuf
* Description    : 读Flash（包含data和code）
* Input          :
* Output         : None
* Return         : None
*******************************************************************************/
void FlashReadBuf(u32 Addr,u8* buf,u16 len)
{
    u16 i;

    for(i=0; i!=len; i++)
    {
        buf[i] = *(u8*)Addr;
        Addr++;
    }
}

/*********************************************************************
 * @fn      FLASH_CFGFLAG_PR
 *
 * @brief   IAP切换和RV-RAM模式切换 标志
 *
 * @param
 *
 * @return  none
 */
void FLASH_CFGFLAG_PR(u32 addr, u8* buf, u8 len )
{
    u16 i;
    u32 addr1;

    addr1 = addr&(~0xFF);
    for(i=0; i<256; i++){
        Fast_Program_Buf[i] = *(u8*)(addr1+i);
    }

    for(i=0; i<len; i++){
        Fast_Program_Buf[i+addr-addr1] = buf[i];
    }

    /* FLASH OP */
    FLASH_Unlock_Fast();
    FLASH_ErasePage_Fast(addr1);
    FLASH_ProgramPage_Fast(addr1, (u32*)Fast_Program_Buf);
    FLASH_Lock_Fast();
}

/*******************************************************************************
* Function Name  : Flash_Modify
* Description    : 用于判断APP是否正常
* Input          : None
* Output         : None
* Return         : None
*******************************************************************************/
void Flash_Modify(void)
{

    FlashReadBuf( ADDR_IAP_APP, Status, 6 );//读取flash，判断参数字是否正确
    if( (Status[0]=='@')&&(Status[1]=='W')&&(Status[2]=='C')&&(Status[3]=='H')&&(Status[5]==0x5A)&&((*(u32*)0x08002050) == 0x12345678)&&((*(u32*)0x08002054) == 0x39373533)){//此时APP正常，跳回APP
        PR("jump APP\r\n");
        Delay_Ms( 50 );
        IAP_2_APP();//跳转至APP程序区

    }

    if(Flag_verifysuc){//下载校验成功后IAP/APP的flash地址写成0x5A
        Flag_verifysuc=0;
        Status[5] = 0x5A;
        FLASH_CFGFLAG_PR(ADDR_IAP_APP, Status, 6);
    }
}

/*******************************************************************************
* Function Name  : LinkStatus
* Description    : ARM/RV模式，IAP/APP模式的状态的写入flash中，
* Output         : None
* Return         : None
*******************************************************************************/
void LinkStatus( void )
{
    u8 i;
    FlashReadBuf(ADDR_IAP_APP, Status, 6);//读取flash，判断参数字是否正确
    for(i=0;i<4;i++){   //判断是否首次写入
        if(Status[i]!=Verify[i]){
            Flag_firstIAP=1;
            break;
        }
        else
            Flag_firstIAP=0;
    }

    if(Flag_firstIAP){  //首次将APP判断flash位写入0x5A (首次下载，APP和IAP一起下载进去)
        for(i=0;i<4;i++){
            Status[i]=Verify[i];
        }
        Status[5] = 0x5A;  //APP
        FLASH_CFGFLAG_PR(ADDR_IAP_APP, Status, 6);
        Flag_firstIAP=0;
    }
}

/*********************************************************************
 * @fn      RecData_Deal
 *
 * @brief   UART-USB deal date
 *
 * @return  ERR_ERROR - ERROR
 *          ERR_SCUESS - SCUESS
 *          ERR_End - End
 */

u8 RecData_Deal(void)
{
     u8 s, Lenth;
     u8 CAN_Tx[2];
     u16 i, j, BinL;

     CAN_Receive_Msg(EP2_Rx_Buffer);

     Lenth = isp_cmd_t->Len;

     switch ( isp_cmd_t->Cmd) {
         case CMD_IAP_START:{
             CRC_Verify_Init();
             FLASH_Unlock_Fast();
             CodeLen = 0;
             Program_addr = ADDR_APP_BUF;
             Verity_addr = ADDR_APP_BUF;
             s = ERR_SCUESS;
             break;
         }
         case CMD_IAP_PROM:{
             for(i=0; i<7; i++){
                 CAN_Receive_Msg(&EP2_Rx_Buffer[8*i+8]);
             }

             Lenth = 60;

             for (i = 0; i < Lenth; i++) {
                 Fast_Program_Buf[CodeLen + i] = isp_cmd_t->data[i];
             }
             CodeLen += Lenth;
             if (CodeLen >= 256) {
                 FLASH_ErasePage_Fast(Program_addr);
                 FLASH_ProgramPage_Fast(Program_addr, (u32*) Fast_Program_Buf);
                 CodeLen -= 256;
                 for (i = 0; i < CodeLen; i++) {
                     Fast_Program_Buf[i] = Fast_Program_Buf[256 + i];
                 }

                 Program_addr += 0x100;

             }
             s = ERR_SCUESS;
             break;
         }
         case CMD_IAP_VERIFY:{
             if (Verity_Star_flag == 0) { //最后1包，256对齐补0xFF 编程
                 Verity_Star_flag = 1;

                 for (i = 0; i < (256 - CodeLen); i++) {
                     Fast_Program_Buf[CodeLen + i] = 0xff;
                 }

                 FLASH_ErasePage_Fast(Program_addr);
                 FLASH_ProgramPage_Fast(Program_addr, (u32*) Fast_Program_Buf);
                 CodeLen = 0;
             }
             for(i=0; i<7; i++){
                 CAN_Receive_Msg(&EP2_Rx_Buffer[8*i+8]);
             }

             Lenth = 60;
             s = ERR_SCUESS;
             for (i = 0; i < Lenth; i++) {
                 if (isp_cmd_t->data[i] != *(u8*) (Verity_addr + i)) {
                     s = ERR_ERROR;
                     break;
                 }
             }

             Verity_addr += Lenth;

             break;
         }
         case CMD_IAP_ALL_VERIFY:{
             Verity_Star_flag = 0;

             Bin_Len = *(u16*)&isp_cmd_t->Rev[0];
             CRC_RxVy = *(u32*)&isp_cmd_t->data[0];

             Verity_addr = ADDR_APP_BUF;

             for(i=0; i<(Bin_Len/4); i++){
                 CRC->DATAR = *(u32*)(Verity_addr + 4*i);
             }
             s = ERR_SCUESS;
             if(CRC->DATAR != CRC_RxVy){
                 s = ERR_ERROR;
             }

             break;
         }
         case CMD_IAP_BufToApp:{
             Program_addr = ADDR_APP;

             if(Bin_Len%256) BinL = Bin_Len/256 + 1;
             else BinL = Bin_Len/256;

             s = ERR_SCUESS;
             for(j=0; j<BinL; j++){

                 //PG
                 for(i=0; i<256; i++){
                     Fast_Program_Buf[i] = *(u8*)(Program_addr + ADDR_APP_BUF - ADDR_APP + i);
                 }

                 FLASH_ErasePage_Fast(Program_addr);
                 FLASH_ProgramPage_Fast(Program_addr, (u32*) Fast_Program_Buf);

                 //Verify
                  for(i=0; i<256; i++){
                      if(*(u8*)(Program_addr+i) != *(u8*)(Program_addr + ADDR_APP_BUF - ADDR_APP + i)){
                          s = ERR_ERROR;
                          break;
                      }
                  }

                 Program_addr += 256;

                 if(s == ERR_ERROR) break;  //Verify ERR
             }

             break;
         }
         case CMD_IAP_END:{
             FLASH_Lock_Fast();
             Flag_verifysuc = 1;
             Flash_Modify();
             NVIC_SystemReset();
             s = ERR_End;
             break;
         }
         default:{
             s = ERR_ERROR;
             break;
         }
     }

     CAN_Tx[0] = 0;
     if(s == ERR_ERROR){
         CAN_Tx[1] = 1;
     }
     else if(s == ERR_SCUESS){
         CAN_Tx[1] = 0;
     }

     if(s != ERR_End){
         CAN_Send_Msg(CAN_Tx, 2);
     }

     return s;
 }

/*********************************************************************
 * @fn      LED_Init
 *
 * @brief
 *
 * @return
 *
 *
 */
void LED_Init(void)
{

    GPIO_InitTypeDef  GPIO_InitStructure;

    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);

    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0|GPIO_Pin_1;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_Init(GPIOA, &GPIO_InitStructure);

    GPIO_SetBits(GPIOA,GPIO_Pin_0|GPIO_Pin_1);

}
